package io.xjar.zip;

import java.io.Closeable;
import io.xjar.XKit;
import org.apache.commons.compress.archivers.ArchiveEntry;
import io.xjar.XUnclosedOutputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import io.xjar.key.XKey;
import io.xjar.XEntryFilter;
import io.xjar.XEncryptor;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import io.xjar.XEntryEncryptor;

public class XZipEncryptor extends XEntryEncryptor<ZipArchiveEntry> implements XEncryptor
{
    private final int level;
    
    public XZipEncryptor(final XEncryptor xEncryptor) {
        this(xEncryptor, null);
    }
    
    public XZipEncryptor(final XEncryptor xEncryptor, final XEntryFilter<ZipArchiveEntry> filter) {
        this(xEncryptor, -1, filter);
    }
    
    public XZipEncryptor(final XEncryptor xEncryptor, final int level) {
        this(xEncryptor, level, null);
    }
    
    public XZipEncryptor(final XEncryptor xEncryptor, final int level, final XEntryFilter<ZipArchiveEntry> filter) {
        super(xEncryptor, (XEntryFilter)filter);
        this.level = level;
    }
    
    public void encrypt(final XKey key, final File src, final File dest) throws IOException {
        try (FileInputStream fis = new FileInputStream(src);
             FileOutputStream fos = new FileOutputStream(dest)) {
            this.encrypt(key, fis, fos);
        }
    }
    
    public void encrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        ZipArchiveInputStream zis = null;
        ZipArchiveOutputStream zos = null;
        try {
            zis = new ZipArchiveInputStream(in);
            zos = new ZipArchiveOutputStream(out);
            zos.setLevel(this.level);
            final XUnclosedOutputStream nos = new XUnclosedOutputStream((OutputStream)zos);
            ZipArchiveEntry entry;
            while ((entry = zis.getNextZipEntry()) != null) {
                if (entry.isDirectory()) {
                    continue;
                }
                zos.putArchiveEntry((ArchiveEntry)new ZipArchiveEntry(entry.getName()));
                final XEncryptor encryptor = (XEncryptor)(this.filtrate((Object)entry) ? this : this.xNopEncryptor);
                try (OutputStream eos = encryptor.encrypt(key, (OutputStream)nos)) {
                    XKit.transfer((InputStream)zis, eos);
                }
                zos.closeArchiveEntry();
            }
            zos.finish();
        }
        finally {
            XKit.close((Closeable)zis);
            XKit.close((Closeable)zos);
        }
    }
}
